/* 
 * $Id: ViewHandlerImpl.java,v 1.77 2006/06/20 17:07:12 edburns Exp $ 
 */ 


/*
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the License at
 * https://javaserverfaces.dev.java.net/CDDL.html or
 * legal/CDDLv1.0.txt. 
 * See the License for the specific language governing
 * permission and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL
 * Header Notice in each file and include the License file
 * at legal/CDDLv1.0.txt.    
 * If applicable, add the following below the CDDL Header,
 * with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * [Name of File] [ver.__] [Date]
 * 
 * Copyright 2005 Sun Microsystems Inc. All Rights Reserved
 */


// ViewHandlerImpl.java 

package com.sun.faces.application;

import javax.faces.FacesException;
import javax.faces.FactoryFinder;
import javax.faces.application.StateManager;
import javax.faces.application.ViewHandler;
import javax.faces.component.UIViewRoot;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.RenderKit;
import javax.faces.render.RenderKitFactory;
import javax.faces.render.ResponseStateManager;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.jstl.core.Config;

import java.io.IOException;
import java.io.Writer;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sun.faces.RIConstants;
import com.sun.faces.config.WebConfiguration;
import com.sun.faces.config.WebConfiguration.WebContextInitParameter;
import com.sun.faces.io.FastStringWriter;
import com.sun.faces.util.DebugUtil;
import com.sun.faces.util.MessageUtils;
import com.sun.faces.util.Util;

/**
 * <B>ViewHandlerImpl</B> is the default implementation class for ViewHandler.
 *
 * @version $Id: ViewHandlerImpl.java,v 1.77 2006/06/20 17:07:12 edburns Exp $
 * @see javax.faces.application.ViewHandler
 */
public class ViewHandlerImpl extends ViewHandler {

    // Log instance for this class
    private static Logger logger = Util.getLogger(Util.FACES_LOGGER
                                                  + Util.APPLICATION_LOGGER);

    // Attribute KEY for context found after <f:view>
    private static final String AFTER_VIEW_CONTENT = RIConstants.FACES_PREFIX+
                                                     "AFTER_VIEW_CONTENT";
    
    // Size of the buffer used to process the response   
    private int bufSize = -1;

    public ViewHandlerImpl() {
        if (logger.isLoggable(Level.FINE)) {
            logger.log(Level.FINE,"Created ViewHandler instance ");
        }
    }


    /**
     * @see ViewHandler#renderView(javax.faces.context.FacesContext, javax.faces.component.UIViewRoot)     
     */
    public void renderView(FacesContext context,
            UIViewRoot viewToRender) throws IOException,
            FacesException {
        
        // suppress rendering if "rendered" property on the component is
        // false
        if (!viewToRender.isRendered()) {
            return;
        }
        
        ExternalContext extContext = context.getExternalContext();
        ServletRequest request = (ServletRequest) extContext.getRequest();
        ServletResponse response = (ServletResponse) extContext.getResponse();
        
        try {
            if (executePageToBuildView(context, viewToRender)) {
                response.flushBuffer(); 
                ApplicationAssociate.getInstance(extContext).responseRendered();
                return;
            }
        } catch (IOException e) {
            throw new FacesException(e);
        }
        
        if (logger.isLoggable(Level.FINE)) {
            logger.log(Level.FINE, "Completed building view for : \n" +
                    viewToRender.getViewId());
        }
        if (logger.isLoggable(Level.FINEST)) {
            logger.log(Level.FINEST, "+=+=+=+=+=+= Printout for " + viewToRender.getViewId() + " about to render.");
            DebugUtil.printTree(viewToRender, logger, Level.FINEST);
        }
        
        // set up the ResponseWriter
        
        RenderKitFactory renderFactory = (RenderKitFactory)
        FactoryFinder.getFactory(FactoryFinder.RENDER_KIT_FACTORY);
        RenderKit renderKit =
                renderFactory.getRenderKit(context, viewToRender.getRenderKitId());        
                
        ResponseWriter oldWriter = context.getResponseWriter();
        initBuffSize(context);

        WriteBehindStringWriter strWriter = 
              new WriteBehindStringWriter(context, bufSize);
        ResponseWriter newWriter;
        if (null != oldWriter) {
            newWriter = oldWriter.cloneWithWriter(strWriter);
        } else {
            newWriter = renderKit.createResponseWriter(strWriter, null,
                    request.getCharacterEncoding());            
        }
        context.setResponseWriter(newWriter);
        
        newWriter.startDocument();
        
        doRenderView(context, viewToRender);
        
        newWriter.endDocument();
        
        // replace markers in the body content and write it to response.

        ResponseWriter responseWriter;
        if (null != oldWriter) {
            responseWriter = oldWriter.cloneWithWriter(response.getWriter());
        } else {
            responseWriter = newWriter.cloneWithWriter(response.getWriter());
        }
        context.setResponseWriter(responseWriter);
        
        if (logger.isLoggable(Level.FINE)) {
            logger.fine("Size of response for view '" + viewToRender + "':" 
                        + strWriter.length());
        }
        strWriter.flushToWriter(responseWriter);
                        
        if (null != oldWriter) {
            context.setResponseWriter(oldWriter);
        }
        
        // write any AFTER_VIEW_CONTENT to the response
        writeAfterViewContent(extContext, response);

    }

    private void initBuffSize(FacesContext context) {
        if (bufSize == -1) {
            synchronized (this) {
                if (bufSize == -1) {
                    WebConfiguration webConfig =
                          WebConfiguration
                                .getInstance(context.getExternalContext());
                    try {
                        bufSize = Integer
                              .parseInt(webConfig.getContextInitParameter(
                                    WebContextInitParameter.ResponseBufferSize));
                    } catch (NumberFormatException nfe) {
                        bufSize = Integer
                              .parseInt(WebContextInitParameter.ResponseBufferSize.getDefaultValue());
                    }
                }
            }
        }
    }


    /**
     * <p>This is a separate method to account for handling the content
     * after the view tag.</p>
     *
     * <p>Create a new ResponseWriter around this response's Writer.
     * Set it into the FacesContext, saving the old one aside.</p>
     *
     * <p>call encodeBegin(), encodeChildren(), encodeEnd() on the
     * argument <code>UIViewRoot</code>.</p>
     *
     * <p>Restore the old ResponseWriter into the FacesContext.</p>
     *
     * <p>Write out the after view content to the response's writer.</p>
     *
     * <p>Flush the response buffer, and remove the after view content
     * from the request scope.</p>
     * 
     * @param context the <code>FacesContext</code> for the current request
     * @param viewToRender the view to render
     * @throws IOException if an error occurs rendering the view to the client
     */
    private void doRenderView(FacesContext context,
                              UIViewRoot viewToRender) 
    throws IOException, FacesException {
        ExternalContext extContext = context.getExternalContext();

        ApplicationAssociate associate =
              ApplicationAssociate.getInstance(extContext);

        if (null != associate) {
            associate.responseRendered();
        }

        if (logger.isLoggable(Level.FINE)) {
            logger.log(Level.FINE,
                       "About to render view " + viewToRender.getViewId());
        }

        viewToRender.encodeAll(context);
    }


    /**
     * @see ViewHandler#restoreView(javax.faces.context.FacesContext, String)     
     */
    public UIViewRoot restoreView(FacesContext context, String viewId) {
        if (context == null) {
            String message = MessageUtils.getExceptionMessageString
                  (MessageUtils.NULL_PARAMETERS_ERROR_MESSAGE_ID, "context");
            throw new NullPointerException(message);
        }              

        String renderKitId = getRenderkitId(context);
        UIViewRoot viewRoot = Util.getStateManager(context).restoreView(context,
                                                             viewId,
                                                             renderKitId);
        
        if (logger.isLoggable(Level.FINEST)) {
            logger.log(Level.FINEST,
                       "+=+=+=+=+=+= Restored View Printout for " + viewId);
            DebugUtil.printTree(viewRoot, logger, Level.FINEST);
        }


        return viewRoot;
    }


    /**
     * @see ViewHandler#createView(javax.faces.context.FacesContext, String)     
     */
    public UIViewRoot createView(FacesContext context, String viewId) {
        if (context == null) {
            String message = MessageUtils.getExceptionMessageString
                    (MessageUtils.NULL_PARAMETERS_ERROR_MESSAGE_ID, "context");
            throw new NullPointerException(message);
        }

        Locale locale = null;
        String renderKitId = null;

        // use the locale from the previous view if is was one which will be
        // the case if this is called from NavigationHandler. There wouldn't be 
        // one for the initial case.
        if (context.getViewRoot() != null) {
            locale = context.getViewRoot().getLocale();
            renderKitId = context.getViewRoot().getRenderKitId();
        }
        UIViewRoot result = (UIViewRoot)
                context.getApplication().createComponent(UIViewRoot.COMPONENT_TYPE);
        result.setViewId(viewId);

        if (logger.isLoggable(Level.FINE)) {
            logger.log(Level.FINE, "Created new view for " + viewId);
        }
        
        // if there was no locale from the previous view, calculate the locale 
        // for this view.
        if (locale == null) {
            locale =
                context.getApplication().getViewHandler().calculateLocale(
                    context);
            if (logger.isLoggable(Level.FINE)) {
                logger.fine("Locale for this view as determined by calculateLocale "
                            + locale.toString());
            }
        } else {
            if (logger.isLoggable(Level.FINE)) {
                logger.fine("Using locale from previous view "
                            + locale.toString());
            }
        }

        if (renderKitId == null) {
            renderKitId =
                context.getApplication().getViewHandler().calculateRenderKitId(
                    context);
           if (logger.isLoggable(Level.FINE)) {
               logger.fine(
               "RenderKitId for this view as determined by calculateRenderKitId "
               + renderKitId);
            }
        } else {
            if (logger.isLoggable(Level.FINE)) {
                logger.fine("Using renderKitId from previous view "
                            + renderKitId);
            }
        }

        result.setLocale(locale);
        result.setRenderKitId(renderKitId);

        return result;
    }

    /**
     * Execute the target view.  If the HTTP status code range is
     * not 2xx, then return true to indicate the response should be
     * immediately flushed by the caller so that conditions such as 404
     * are properly handled.
     * @param context the <code>FacesContext</code> for the current request
     * @param viewToExecute the view to build
     * @return <code>true</code> if the response should be immediately flushed 
     *  to the client, otherwise <code>false</code> 
     * @throws IOException if an error occurs executing the page    
     */
    private boolean executePageToBuildView(FacesContext context,
                                        UIViewRoot viewToExecute) 
    throws IOException {

        if (null == context) {
            String message = MessageUtils.getExceptionMessageString
                    (MessageUtils.NULL_PARAMETERS_ERROR_MESSAGE_ID, "context");
            throw new NullPointerException(message);
        }
        if (null == viewToExecute) {
            String message = MessageUtils.getExceptionMessageString
                    (MessageUtils.NULL_PARAMETERS_ERROR_MESSAGE_ID, "viewToExecute");
            throw new NullPointerException(message);
        }

        String mapping = Util.getFacesMapping(context);
        String requestURI = 
              updateRequestURI(viewToExecute.getViewId(), mapping);
        
        if (mapping.equals(requestURI)) {
            // The request was to the FacesServlet only - no path info
            // on some containers this causes a recursion in the 
            // RequestDispatcher and the request appears to hang.
            // If this is detected, return status 404
            HttpServletResponse response = (HttpServletResponse) 
                  context.getExternalContext().getResponse();
            response.sendError(HttpServletResponse.SC_NOT_FOUND);
            return true;
        }
        if (logger.isLoggable(Level.FINE)) {
            logger.fine("About to execute view " + requestURI);
        }
                       
        ExternalContext extContext = context.getExternalContext();

        // update the JSTL locale attribute in request scope so that JSTL
        // picks up the locale from viewRoot. This attribute must be updated
        // before the JSTL setBundle tag is called because that is when the
        // new LocalizationContext object is created based on the locale.
        // PENDING: this only works for servlet based requests
        if (extContext.getRequest()
        instanceof ServletRequest) {
            Config.set((ServletRequest)
            extContext.getRequest(),
                       Config.FMT_LOCALE, context.getViewRoot().getLocale());
        }
        if (logger.isLoggable(Level.FINE)) {
            logger.fine("Before dispacthMessage to newViewId " 
                        + viewToExecute.getViewId());
        }

        // save the original response
        Object originalResponse = extContext.getResponse();

        // replace the response with our wrapper
        ViewHandlerResponseWrapper wrapped = 
              new ViewHandlerResponseWrapper(
                    (HttpServletResponse)extContext.getResponse());
        extContext.setResponse(wrapped);

        // build the view by executing the page
        extContext.dispatch(requestURI);        
        
        if (logger.isLoggable(Level.FINE)) {
            logger.fine("After dispacthMessage to newViewId " 
                        + viewToExecute.getViewId());
        }
        
        // replace the original response
        extContext.setResponse(originalResponse);
        
        // Follow the JSTL 1.2 spec, section 7.4,  
        // on handling status codes on a forward
        if (wrapped.getStatus() < 200 || wrapped.getStatus() > 299) {  
            // flush the contents of the wrapper to the response
            // this is necessary as the user may be using a custom 
            // error page - this content should be propagated
            wrapped.flushContentToWrappedResponse();
            return true;            
        }               
        
        // Put the AFTER_VIEW_CONTENT into request scope
        // temporarily
        if (wrapped.isBytes()) {
            extContext.getRequestMap().put(AFTER_VIEW_CONTENT,
                                           wrapped.getBytes());
        } else if (wrapped.isChars()) {
            extContext.getRequestMap().put(AFTER_VIEW_CONTENT,
                                           wrapped.getChars());
        }

        return false;
        
    }


    public Locale calculateLocale(FacesContext context) {

        if (context == null) {
            String message = MessageUtils.getExceptionMessageString
                (MessageUtils.NULL_PARAMETERS_ERROR_MESSAGE_ID, "context");
            throw new NullPointerException(message);
        }

        Locale result = null;
        // determine the locales that are acceptable to the client based on the 
        // Accept-Language header and the find the best match among the 
        // supported locales specified by the client.
        Iterator<Locale> locales = context.getExternalContext().getRequestLocales();
        while (locales.hasNext()) {
            Locale perf = locales.next();
            result = findMatch(context, perf);
            if (result != null) {
                break;
            }
        }
        // no match is found.
        if (result == null) {
            if (context.getApplication().getDefaultLocale() == null) {
                result = Locale.getDefault();
            } else {
                result = context.getApplication().getDefaultLocale();
            }
        }
        return result;
    }


    public String calculateRenderKitId(FacesContext context) {

        if (context == null) {
            String message = MessageUtils.getExceptionMessageString
                (MessageUtils.NULL_PARAMETERS_ERROR_MESSAGE_ID, "context");
            throw new NullPointerException(message);
        }       

        Map<String,String> requestParamMap = context.getExternalContext()
            .getRequestParameterMap();
        String result = requestParamMap.get(
            ResponseStateManager.RENDER_KIT_ID_PARAM);

        if (result == null) {
            if (null ==
                (result = context.getApplication().getDefaultRenderKitId())) {
                result = RenderKitFactory.HTML_BASIC_RENDER_KIT;
            }
        }
        return result;
    }


    /**
     * Attempts to find a matching locale based on <code>pref</code> and
     * list of supported locales, using the matching algorithm
     * as described in JSTL 8.3.2.
     * @param context the <code>FacesContext</code> for the current request
     * @param pref the preferred locale
     * @return the Locale based on pref and the matching alogritm specified
     *  in JSTL 8.3.2
     */
    protected Locale findMatch(FacesContext context, Locale pref) {
        Locale result = null;
        Iterator<Locale> it = context.getApplication().getSupportedLocales();
        while (it.hasNext()) {
            Locale supportedLocale = it.next();

            if (pref.equals(supportedLocale)) {
                // exact match
                result = supportedLocale;
                break;
            } else {
                // Make sure the preferred locale doesn't have country
                // set, when doing a language match, For ex., if the
                // preferred locale is "en-US", if one of supported
                // locales is "en-UK", even though its language matches
                // that of the preferred locale, we must ignore it.
                if (pref.getLanguage().equals(supportedLocale.getLanguage()) &&
                    supportedLocale.getCountry().equals("")) {
                    result = supportedLocale;
                }
            }
        }
        // if it's not in the supported locales,
        if (null == result) {
            Locale defaultLocale = context.getApplication().getDefaultLocale();
            if (defaultLocale != null) {
                if ( pref.equals(defaultLocale)) {
                    // exact match
                    result = defaultLocale;
                } else {
                    // Make sure the preferred locale doesn't have country
                    // set, when doing a language match, For ex., if the
                    // preferred locale is "en-US", if one of supported
                    // locales is "en-UK", even though its language matches
                    // that of the preferred locale, we must ignore it.
                    if (pref.getLanguage().equals(defaultLocale.getLanguage()) &&
                        defaultLocale.getCountry().equals("")) {
                        result = defaultLocale;
                    }
                }
            }
        }

        return result;
    }


    public void writeState(FacesContext context) throws IOException {
        if (context == null) {
           String message = MessageUtils.getExceptionMessageString
                (MessageUtils.NULL_PARAMETERS_ERROR_MESSAGE_ID, "context");
            throw new NullPointerException(message);
        }

        if (logger.isLoggable(Level.FINE)) {
            logger.fine("Begin writing marker for viewId " +
                        context.getViewRoot().getViewId());
        }

        context.getResponseWriter().writeText(
                    RIConstants.SAVESTATE_FIELD_MARKER, null);
        if (logger.isLoggable(Level.FINE)) {
            logger.fine("End writing marker for viewId " +
                        context.getViewRoot().getViewId());
        }

    }


    public String getActionURL(FacesContext context, String viewId) {

        if (context == null) {
            String message = MessageUtils.getExceptionMessageString
                (MessageUtils.NULL_PARAMETERS_ERROR_MESSAGE_ID, "context");
            throw new NullPointerException(message);
        }
        if (viewId == null) {
            String message = MessageUtils.getExceptionMessageString
                (MessageUtils.NULL_PARAMETERS_ERROR_MESSAGE_ID, "viewId");
            throw new NullPointerException(message);
        }

        if (viewId.charAt(0) != '/') {
            String message = 
                  MessageUtils.getExceptionMessageString(
                        MessageUtils.ILLEGAL_VIEW_ID_ID,
                        viewId);
            if (logger.isLoggable(Level.SEVERE)) {
                logger.log(Level.SEVERE, "jsf.illegal_view_id_error", viewId);
            }
            throw new IllegalArgumentException(message);
        }

        // Acquire the context path, which we will prefix on all results
        String contextPath =
            context.getExternalContext().getRequestContextPath();

        // Acquire the mapping used to execute this request (if any)
        String mapping = Util.getFacesMapping(context);

        // If no mapping can be identified, just return a server-relative path
        if (mapping == null) {
            return contextPath + viewId;
        }

        // Deal with prefix mapping
        if (Util.isPrefixMapped(mapping)) {
            if (mapping.equals("/*")) {
                return contextPath + viewId;
            } else {
                return contextPath + mapping + viewId;
            }
        }

        // Deal with extension mapping
        int period = viewId.lastIndexOf(".");
        if (period < 0) {
            return contextPath + viewId + mapping;
        } else if (!viewId.endsWith(mapping)) {
            return contextPath + viewId.substring(0, period) + mapping;
        } else {
            return contextPath + viewId;
        }

    }


    public String getResourceURL(FacesContext context, String path) {

        if (path.startsWith("/")) {
            return context.getExternalContext().getRequestContextPath() + path;
        } else {
            return (path);
        }

    }    
    

    /**
     * <p>if the specified mapping is a prefix mapping, and the provided 
     * request URI (usually the value from <code>ExternalContext.getRequestServletPath()</code>)
     * starts with <code>mapping + '/'</code>, prune the mapping from the
     * URI and return it, otherwise, return the original URI. 
     * @param uri the servlet request path
     * @param mapping the FacesServlet mapping used for this request
     * @return the URI without additional FacesServlet mappings
     * @since 1.2
     */
    private String updateRequestURI(String uri, String mapping) {
        
        if (!Util.isPrefixMapped(mapping)) {
            return uri;
        } else {
            int length = mapping.length() + 1;
            StringBuilder builder = new StringBuilder(length);
            builder.append(mapping).append('/');
            String mappingMod = builder.toString();
            boolean logged = false;
            while (uri.startsWith(mappingMod)) {
                if (!logged && logger.isLoggable(Level.WARNING)) {
                    logged = true;
                    logger.log(Level.WARNING,
                               "jsf.viewhandler.requestpath.recursion",
                               new Object[] {uri, mapping});
                }
                uri = uri.substring(length - 1);
            }
            return uri;
        }
    }    

    
    /**
     * <p>Special handling of getRenderKitId for decorated
     * implementations.</p>
     * @param context the <code>FacesContext</code> for the current request
     * @return the calculated RenderKit ID
     */
    private String getRenderkitId(FacesContext context) {
        // this is necessary to allow decorated impls.        
        return context.getApplication().getViewHandler()
              .calculateRenderKitId(context);
    }
    
    
     private void writeAfterViewContent(ExternalContext extContext,
                                       ServletResponse response)
    throws IOException {
        Object content = extContext.getRequestMap().get(AFTER_VIEW_CONTENT);
        assert(null != content);
        if (content instanceof char []) {
            response.getWriter().write((char []) content);
        } else if (content instanceof byte []) {
            response.getWriter().write(new String((byte[]) content));
        } else {
            assert(false);
        }

        response.flushBuffer();

        // remove the AFTER_VIEW_CONTENT from the view root
        extContext.getRequestMap().remove(AFTER_VIEW_CONTENT);
    }
   
    
    // ----------------------------------------------------------- Inner Classes

    /**
     * <p>Handles writing the response from the dispatched request
     * and replaces any state markers with the actual
     * state supplied by the <code>StateManager</code>.
     */
    private static final class WriteBehindStringWriter extends FastStringWriter {
                        
        // length of the state marker
        private static final int STATE_MARKER_LEN = 
              RIConstants.SAVESTATE_FIELD_MARKER.length();
                        
        // the context for the current request
        private final FacesContext context;
        
        // char buffer
        private final char[] buf;
        
        // buffer length
        private final int bufSize;        
        

        /**
         * <p>Create a new <code>WriteBehindStringWriter</code> for the current
         * request with an initial capacity.</p>
         * @param context the <code>FacesContext</code> for the current request
         * @param initialCapcity the StringBuilder's initial capacity
         */
        public WriteBehindStringWriter(FacesContext context, int initialCapcity) {
            super(initialCapcity);         
            this.context = context;      
            bufSize = initialCapcity;
            buf = new char[bufSize];
        }

        /**
         * <p> Write directly from our StringBuilder to the provided
         * writer.</p>
         * @param writer where to write
         * @throws IOException if an error occurs
         */
        public void flushToWriter(Writer writer) throws IOException {
            StateManager stateManager = Util.getStateManager(context);
            Object stateToWrite = stateManager.saveView(context);
            int totalLen = builder.length();
            int pos = 0;
            int tildeIdx = getNextDelimiterIndex(pos);
            while (pos < totalLen) {
                if (tildeIdx != -1) {
                    if (tildeIdx > pos && (tildeIdx - pos) > bufSize) {
                        // theres enough content before the first ~ 
                        // to fill the entire buffer
                        builder.getChars(pos, (pos + bufSize), buf, 0);
                        writer.write(buf);
                        pos += bufSize;
                    } else {
                        // write all content up to the first '~'
                        builder.getChars(pos, tildeIdx, buf, 0);
                        int len = (tildeIdx - pos);
                        writer.write(buf, 0, len);                       
                        // now check to see if the state saving string is
                        // at the begining of pos, if so, write our 
                        // state out.                               
                        if (builder.indexOf(
                              RIConstants.SAVESTATE_FIELD_MARKER, 
                              pos) == tildeIdx) {
                            stateManager.writeState(context, stateToWrite);
                        }
                        
                        // push us past the last '~' at the end of the marker
                        pos += (len + STATE_MARKER_LEN);
                        tildeIdx = getNextDelimiterIndex(pos);
                    }
                } else {
                    // we've written all of the state field markers.
                    // finish writing content
                    if (totalLen - pos > bufSize) {
                        // there's enough content to fill the buffer
                        builder.getChars(pos, (pos + bufSize), buf, 0);
                        writer.write(buf);
                        pos += bufSize;
                    } else {
                        // we're near the end of the response
                        builder.getChars(pos, totalLen, buf, 0);
                        int len = (totalLen - pos);
                        writer.write(buf, 0, len);
                        pos += (len + 1);                      
                    }
                }
            }
        }

        /**         
         * @return return the length of the underlying 
         * <code>StringBuilder</code>.
         */
        public int length() {
            return builder.length();
        }

        /**
         * <p>Get the next `~' from the StringBuilder.</p>
         * @param offset the offset from where to search from
         * @return the index of the first '~' from the specified
         *  offset
         */
        private int getNextDelimiterIndex(int offset) {
            return builder.indexOf(RIConstants.SAVESTATE_FIELD_DELIMITER, 
                                   offset);            
        }
        
       
    }
}
